/*
  $Header: /home/richard/myntp/chrony/chrony-1.0/RCS/candm.h,v 1.25 1998/05/29 05:15:51 richard Exp $

  =======================================================================

  chronyd/chronyc - Programs for keeping computer clocks accurate.

  Copyright (C) 1997-1998 Richard P. Curnow <richard@curnow.demon.co.uk>
  All rights reserved.

  For conditions of use, refer to the file LICENCE.

  =======================================================================

  Definitions for the network protocol used for command and monitoring
  of the timeserver.

  */

#ifndef GOT_CANDM_H
#define GOT_CANDM_H

#include <sys/time.h>

/* This is the default port to use for CANDM, if no alternative is
   defined */
#define DEFAULT_CANDM_PORT 323

/* Request codes */
#define REQ_NULL 0
#define REQ_ONLINE 1
#define REQ_OFFLINE 2
#define REQ_BURST 3
#define REQ_MODIFY_MINPOLL 4
#define REQ_MODIFY_MAXPOLL 5
#define REQ_DUMP 6
#define REQ_MODIFY_MAXDELAY 7
#define REQ_MODIFY_MAXDELAYRATIO 8
#define REQ_MODIFY_MAXUPDATESKEW 9
#define REQ_LOGON 10
#define REQ_SETTIME 11
#define REQ_LOCAL 12
#define REQ_MANUAL 13
#define REQ_N_SOURCES 14
#define REQ_SOURCE_DATA 15
#define REQ_REKEY 16
#define REQ_ALLOW 17
#define REQ_ALLOWALL 18
#define REQ_DENY 19
#define REQ_DENYALL 20
#define REQ_CMDALLOW 21
#define REQ_CMDALLOWALL 22
#define REQ_CMDDENY 23
#define REQ_CMDDENYALL 24
#define REQ_ACCHECK 25
#define REQ_CMDACCHECK 26
#define REQ_ADD_SERVER 27
#define REQ_ADD_PEER 28
#define REQ_DEL_SOURCE 29
#define REQ_WRITERTC 30
#define REQ_DFREQ 31
#define REQ_DOFFSET 32
#define REQ_TRACKING 33
#define REQ_SOURCESTATS 34
#define REQ_RTCREPORT 35
#define REQ_TRIMRTC 36
#define N_REQUEST_TYPES 37

/* Special utoken value used to log on with first exchange being the
   password.  (This time value has long since gone by) */
#define SPECIAL_UTOKEN 0x10101010

typedef struct {
  unsigned long mask;
  unsigned long address;
} REQ_Online;

typedef struct {
  unsigned long mask;
  unsigned long address;
} REQ_Offline;

typedef struct {
  unsigned long mask;
  unsigned long address;
  int n_good_samples;
  int n_total_samples;
} REQ_Burst;

typedef struct {
  unsigned long address;
  int new_minpoll;
} REQ_Modify_Minpoll;

typedef struct {
  unsigned long address;
  int new_maxpoll;
} REQ_Modify_Maxpoll;

typedef struct {
  int pad;
} REQ_Dump;

typedef struct {
  unsigned long address;
  long new_max_delay;
} REQ_Modify_Maxdelay;

typedef struct {
  unsigned long address;
  long new_max_delay_ratio;
} REQ_Modify_Maxdelayratio;

typedef struct {
  long new_max_update_skew;
} REQ_Modify_Maxupdateskew;

typedef struct {
  struct timeval ts;
} REQ_Logon;

typedef struct {
  struct timeval ts;
} REQ_Settime;

typedef struct {
  int on_off;
  int stratum;
} REQ_Local;

typedef struct {
  int option;
} REQ_Manual;

typedef struct {
  int pad1;
} REQ_N_Sources;

typedef struct {
  int index;
} REQ_Source_Data;

typedef struct {
  int pad1;
} REQ_Rekey;

typedef struct {
  unsigned long ip;
  int subnet_bits;
} REQ_Allow_Deny;

typedef struct {
  unsigned long ip;
} REQ_Ac_Check;

typedef struct {
  unsigned long ip_addr;
  unsigned long port;
  int minpoll;
  int maxpoll;
  int presend_minpoll;
  int online;
  unsigned long authkey;
  long max_delay;
  long max_delay_ratio;
} REQ_NTP_Source;

typedef struct {
  unsigned long ip_addr;
} REQ_Del_Source;

typedef struct {
  int pad1;
} REQ_WriteRtc;

typedef struct {
  long dfreq;
} REQ_Dfreq;

typedef struct {
  long sec;
  long usec;
} REQ_Doffset;

typedef struct {
  unsigned long pad1;
} REQ_Tracking;

typedef struct {
  unsigned long index;
} REQ_Sourcestats;

typedef struct {
  unsigned long pad1;
} REQ_RTCReport;

typedef struct {
  unsigned long pad1;
} REQ_TrimRTC;

/* ================================================== */

#define PKT_TYPE_CMD_REQUEST 1
#define PKT_TYPE_CMD_REPLY 2

#define PROTO_VERSION_NUMBER 1

/* ================================================== */

typedef struct {
  unsigned char version; /* Protocol version */
  unsigned char pkt_type; /* What sort of packet this is */
  unsigned char res1;
  unsigned char res2;
  unsigned short command; /* Which command is being issued */
  unsigned short attempt; /* How many resends the client has done
                             (count up from zero for same sequence
                             number) */
  unsigned long sequence; /* Client's sequence number */
  unsigned long utoken; /* Unique token per incarnation of daemon */
  unsigned long token; /* Command token (to prevent replay attack) */
  unsigned long auth[4]; /* MD5 authentication of the packet */

  union {
    REQ_Online online;
    REQ_Offline offline;
    REQ_Burst burst;
    REQ_Modify_Minpoll modify_minpoll;
    REQ_Modify_Maxpoll modify_maxpoll;
    REQ_Dump dump;
    REQ_Modify_Maxdelay modify_maxdelay;
    REQ_Modify_Maxdelayratio modify_maxdelayratio;
    REQ_Modify_Maxupdateskew modify_maxupdateskew;
    REQ_Logon logon;
    REQ_Settime settime;
    REQ_Local local;
    REQ_Manual manual;
    REQ_N_Sources n_sources;
    REQ_Source_Data source_data;
    REQ_Rekey rekey;
    REQ_Allow_Deny allow_deny;
    REQ_Ac_Check ac_check;
    REQ_NTP_Source ntp_source;
    REQ_Del_Source del_source;
    REQ_WriteRtc writertc;
    REQ_Dfreq dfreq;
    REQ_Doffset doffset;
    REQ_Tracking tracking;
    REQ_Sourcestats sourcestats;
    REQ_RTCReport rtcreport;
    REQ_TrimRTC trimrtc;
  } data; /* Command specific parameters */

} CMD_Request;

/* ================================================== */
/* Authority codes for command types */

#define PERMIT_OPEN 0
#define PERMIT_LOCAL 1
#define PERMIT_AUTH 2

/* ================================================== */
/* These conversion utilities are used to convert between the internal
   and the 'wire' representation of real quantities */

#define WIRE2REAL(x) ((double) ((long) ntohl(x)) / 65536.0)
#define REAL2WIRE(x) (htonl((long)(0.5 + 65536.0 * (x))))

/* ================================================== */

/* Reply codes */
#define RPY_NULL 1
#define RPY_N_SOURCES 2
#define RPY_SOURCE_DATA 3
#define RPY_OFFSET_CS 4
#define RPY_TRACKING 5
#define RPY_SOURCESTATS 6
#define RPY_RTC 7

/* Status codes */
#define STT_SUCCESS 0
#define STT_FAILED 1
#define STT_UNAUTH 2
#define STT_INVALID 3
#define STT_NOSUCHSOURCE 4
#define STT_INVALIDTS 5
#define STT_NOTENABLED 6
#define STT_BADSUBNET 7
#define STT_ACCESSALLOWED 8
#define STT_ACCESSDENIED 9
#define STT_NOHOSTACCESS 10
#define STT_SOURCEALREADYKNOWN 11
#define STT_TOOMANYSOURCES 12
#define STT_NORTC 13
#define STT_BADRTCFILE 14

typedef struct {
  int pad1;
} RPY_Null;

typedef struct {
  unsigned long n_sources;
} RPY_N_Sources;

#define RPY_SD_MD_CLIENT 0
#define RPY_SD_MD_PEER   1
#define RPY_SD_MD_REF    2

#define RPY_SD_ST_SYNC 0
#define RPY_SD_ST_UNREACH 1
#define RPY_SD_ST_FALSETICKER 2
#define RPY_SD_ST_JITTERY 3
#define RPY_SD_ST_OTHER 4

typedef struct {
  unsigned long ip_addr;
  unsigned short poll;
  unsigned short stratum;
  unsigned short state;
  unsigned short mode;
  unsigned long  since_sample;
  long orig_latest_meas;
  long latest_meas;
  unsigned long latest_meas_err;
  long est_offset;
  unsigned long est_offset_err;
  long resid_freq;
  unsigned long resid_skew;
} RPY_Source_Data;

typedef struct {
  unsigned long ref_id;
  unsigned long stratum;
  unsigned long ref_time_s;
  unsigned long ref_time_us;
  unsigned long current_correction_s;
  unsigned long current_correction_us;
  long freq_ppm;
  long resid_freq_ppm;
  long skew_ppm;
  long root_delay;
  long root_dispersion;
} RPY_Tracking;

typedef struct {
  unsigned long ip_addr;
  unsigned long n_samples;
  unsigned long n_runs;
  unsigned long span_seconds;
  unsigned long sd_us;
  long resid_freq_ppm;
  long skew_ppm;
} RPY_Sourcestats;

typedef struct {
  unsigned long ref_time;
  unsigned short n_samples;
  unsigned short n_runs;
  unsigned long span_seconds;
  long rtc_seconds_fast;
  long rtc_gain_rate_ppm;
} RPY_Rtc;

typedef struct {
  unsigned long centiseconds;
} RPY_Offset_Cs;

typedef struct {
  unsigned char version;
  unsigned char pkt_type;
  unsigned char res1;
  unsigned char res2;
  unsigned short command; /* Which command is being replied to */
  unsigned short reply; /* Which format of reply this is */
  unsigned short status; /* Status of command processing */
  unsigned short number; /* Which packet this is in reply sequence */
  unsigned short total; /* Number of replies to expect in this sequence */
  unsigned short pad1; /* Get up to 4 byte alignment */
  unsigned long sequence; /* Echo of client's sequence number */
  unsigned long utoken; /* Unique token per incarnation of daemon */
  unsigned long token; /* New command token (only if command was successfully
                          authenticated) */
  unsigned long auth[4]; /* MD5 authentication of the packet */

  union {
    RPY_Null null;
    RPY_N_Sources n_sources;
    RPY_Source_Data source_data;
    RPY_Offset_Cs offset_cs;
    RPY_Tracking tracking;
    RPY_Sourcestats sourcestats;
    RPY_Rtc rtc;
  } data; /* Reply specific parameters */

} CMD_Reply;

/* ================================================== */

#endif /* GOT_CANDM_H */
