/*
  $Header: /home/richard/myntp/chrony/chrony-1.0/RCS/logging.c,v 1.6 1998/05/20 06:10:50 richard Exp $

  =======================================================================

  chronyd/chronyc - Programs for keeping computer clocks accurate.

  Copyright (C) 1997-1998 Richard P. Curnow <richard@curnow.demon.co.uk>
  All rights reserved.

  For conditions of use, refer to the file LICENCE.

  =======================================================================

  Module to handle logging of diagnostic information
  */

#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <syslog.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "main.h"
#include "logging.h"
#include "version.h"

/* ================================================== */
/* Flag indicating we have initialised */
static int initialised = 0;

static int is_detached = 0;

/* ================================================== */
/* Init function */

void
LOG_Initialise(void)
{
  initialised = 1;
  return;
}

/* ================================================== */
/* Fini function */

void
LOG_Finalise(void)
{
  if (is_detached) {
    closelog();
  }

  initialised = 0;
  return;
}

/* ================================================== */

void
LOG_Line_Function(LOG_Severity severity, LOG_Facility facility, const char *format, ...)
{
  char buf[2048];
  va_list other_args;
  va_start(other_args, format);
  vsprintf(buf, format, other_args);
  va_end(other_args);
  if (is_detached) {
    switch (severity) {
      case LOGS_INFO:
        syslog(LOG_INFO, "%s", buf);
        break;
      case LOGS_WARN:
        syslog(LOG_WARNING, "%s", buf);
        break;
      case LOGS_ERR:
      default:
        syslog(LOG_ERR, "%s", buf);
        break;
    }
  } else {
    fprintf(stderr, "%s", buf);
  }
  return;
}

/* ================================================== */

volatile void
LOG_Fatal_Function(LOG_Facility facility, const char *format, ...)
{
  char buf[2048];
  va_list other_args;
  va_start(other_args, format);
  vsprintf(buf, format, other_args);
  va_end(other_args);

  if (is_detached) {
    syslog(LOG_CRIT, "Fatal error : %s", buf);
  } else {
    fprintf(stderr, "Fatal error : %s", buf);
  }

  MAI_CleanupAndExit();

  return;
}

/* ================================================== */

void
LOG_Position(const char *filename, int line_number, const char *function_name)
{
  time_t t;
  struct tm stm;
  char buf[64];
  if (!is_detached) {
    /* Don't clutter up syslog with internal debugging info */
    time(&t);
    stm = *gmtime(&t);
    strftime(buf, sizeof(buf), "%d-%H:%M:%S", &stm);
    fprintf(stderr, "%s:%d:(%s)[%s] ", filename, line_number, function_name, buf);
  }
  return;
}

/* ================================================== */

void
LOG_GoDaemon(void)
{
  int pid, fd;

  /* Does this preserve existing signal handlers? */
  pid = fork();

  if (pid < 0) {
    LOG(LOGS_ERR, LOGF_Logging, "Could not detach, fork failed : %s", strerror(errno));
  } else {
    if (pid > 0) {
      exit(0); /* In the parent */
    } else {

      setsid();

      for (fd=0; fd<1024; fd++) {
        close(fd);
      }

      is_detached = 1;

      openlog("chronyd", LOG_PID, LOG_DAEMON);

      LOG(LOGS_INFO, LOGF_Logging, "chronyd version %s starting", PROGRAM_VERSION_STRING);
  
    }
  }
}

/* ================================================== */
